package com.huijy.web.controller.management;

import java.util.List;

import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.huijy.common.annotation.Log;
import com.huijy.common.core.controller.BaseController;
import com.huijy.common.core.domain.AjaxResult;
import com.huijy.common.enums.BusinessType;
import com.huijy.management.domain.Feedback;
import com.huijy.management.service.IFeedbackService;
import com.huijy.common.utils.poi.ExcelUtil;
import com.huijy.common.core.page.TableDataInfo;

/**
 * 建议反馈Controller
 *
 * @author lishuwen
 * @date 2021-11-23
 */
@RestController
@RequestMapping("/management/feedback")
public class FeedbackController extends BaseController {
    @Autowired
    private IFeedbackService feedbackService;

    /**
     * 查询建议反馈列表
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:list')")
    @GetMapping("/list")
    public TableDataInfo list(Feedback feedback) {
        startPage();
        List<Feedback> list = feedbackService.selectFeedbackList(feedback);
        return getDataTable(list);
    }

    /**
     * 导出建议反馈列表
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:export')")
    @Log(title = "建议反馈", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public AjaxResult export(Feedback feedback) {
        List<Feedback> list = feedbackService.selectFeedbackList(feedback);
        ExcelUtil<Feedback> util = new ExcelUtil<Feedback>(Feedback.class);
        return util.exportExcel(list, "建议反馈数据");
    }

    /**
     * 获取建议反馈详细信息
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id) {
        return AjaxResult.success(feedbackService.selectFeedbackById(id));
    }

    /**
     * 新增建议反馈
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:add')")
    @Log(title = "建议反馈", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody Feedback feedback) {
        return toAjax(feedbackService.insertFeedback(feedback));
    }

    /**
     * 修改建议反馈
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:edit')")
    @Log(title = "建议反馈", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody Feedback feedback) {
        return toAjax(feedbackService.updateFeedback(feedback));
    }

    /**
     * 删除建议反馈
     */
    @PreAuthorize("@ss.hasPermi('management:feedback:remove')")
    @Log(title = "建议反馈", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids) {
        return toAjax(feedbackService.deleteFeedbackByIds(ids));
    }
}
